Changed name of project in readme.
[lurk.git] / lurk.el
diff --git a/lurk.el b/lurk.el
index 4d2a92e..e2d175e 100644 (file)
--- a/lurk.el
+++ b/lurk.el
@@ -1,4 +1,4 @@
;;; lurk.el --- Little Uni-buffer iRc Klient -*- lexical-binding:t -*-
 
 ;; Copyright (C) 2021 Tim Vaughan
 
@@ -35,7 +35,7 @@
 ;;
 
 (defgroup lurk nil
-  "Little Uni-buffer iRc Klient."
+  "Little Unibuffer iRc Klient."
   :group 'network)
 
 (defcustom lurk-nick "plugd"
@@ -58,6 +58,9 @@
 (defcustom lurk-show-joins nil
   "Set to non-nil to be notified of joins, parts and quits.")
 
+(defcustom lurk-display-header t
+  "If non-nil, use buffer header to display information on current host and channel.")
+
 ;;; Faces
 ;;
 
@@ -364,6 +367,24 @@ portion of the source component of the message, as LURK doesn't use this.")
 (defvar lurk-input-marker nil
   "Marker for prompt position in LURK buffer.")
 
+(defun lurk-setup-header ()
+  (setq-local header-line-format
+              '(:eval
+                (let ((proc (get-process "lurk")))
+                  (if proc
+                      (concat
+                       "Host: " (car (process-contact proc))
+                       ", Context: "
+                       (if lurk-current-context
+                           (concat
+                            lurk-current-context
+                            " ("
+                            (number-to-string
+                             (length (lurk-get-context-users lurk-current-context)))
+                            " users)")
+                         "Server"))
+                    "No connection")))))
+
 (defun lurk-setup-buffer ()
   (with-current-buffer (get-buffer-create "*lurk*")
     (setq-local scroll-conservatively 1)
@@ -375,17 +396,19 @@ portion of the source component of the message, as LURK doesn't use this.")
         (set-marker lurk-input-marker (point-max))
       (setq lurk-input-marker (point-max-marker)))
     (goto-char (point-max))
-    (lurk-render-prompt)))
+    (lurk-render-prompt)
+    (if lurk-display-header
+        (lurk-setup-header))))
 
 
 ;;; Output formatting and highlighting
 ;;
 
-;; Partially-implemented idea: the face text property can be
-;; a list of faces, applied in order.  By assigning each context
-;; a unique list and keeping track of these in a hash table, we can
-;; easily switch the face corresponding to a particular context
-;; by modifying the elements of this list.
+;; Idea: the face text property can be a list of faces, applied in
+;; order.  By assigning each context a unique list and keeping track
+;; of these in a hash table, we can easily switch the face
+;; corresponding to a particular context by modifying the elements of
+;; this list.
 ;;
 ;; More subtly, we make only the cdrs of this list shared among
 ;; all text of a given context, allowing the cars to be different
@@ -405,6 +428,15 @@ portion of the source component of the message, as LURK doesn't use this.")
       (puthash context facelist lurk-context-facelists))
     facelist))
 
+(defun lurk--fill-strings (col indent &rest strings)
+  (with-temp-buffer
+    (setq buffer-invisibility-spec nil)
+    (let ((fill-column col)
+          (adaptive-fill-regexp (rx-to-string `(= ,indent anychar))))
+      (apply #'insert strings)
+      (fill-region (point-min) (point-max) nil t)
+      (buffer-string))))
+
 (defun lurk-display-string (context prefix &rest strings)
   (with-current-buffer (get-buffer-create "*lurk*")
     (save-excursion
@@ -413,28 +445,27 @@ portion of the source component of the message, as LURK doesn't use this.")
              (old-pos (marker-position lurk-prompt-marker))
              (padded-timestamp (concat (format-time-string "%H:%M ")))
              (padded-prefix (if prefix (concat prefix " ") ""))
-             (adaptive-fill-regexp (rx-to-string
-                                    `(= ,(+ (length padded-timestamp)
-                                            (length padded-prefix))
-                                        anychar)))
-             (fill-column 80)
              (context-atom (if context (intern context) nil)))
         (insert-before-markers
-         (propertize padded-timestamp
-                     'face 'lurk-timestamp
-                     'read-only t
-                     'context context
-                     'invisible context-atom)
-         (propertize padded-prefix
-                     'read-only t
-                     'context context
-                     'invisible context-atom)
-         (propertize (concat (lurk-buttonify-urls (apply #'concat strings)) "\n")
-                     'face (lurk-get-context-facelist context)
-                     'read-only t
-                     'context context
-                     'invisible context-atom))
-        (fill-region old-pos lurk-prompt-marker nil t)))))
+         (lurk--fill-strings
+          80
+          (+ (length padded-timestamp)
+             (length padded-prefix))
+          (propertize padded-timestamp
+                      'face 'lurk-timestamp
+                      'read-only t
+                      'context context
+                      'invisible context-atom)
+          (propertize padded-prefix
+                      'read-only t
+                      'context context
+                      'invisible context-atom)
+          (lurk-add-formatting
+           (propertize (concat (apply #'lurk-buttonify-urls strings) "\n")
+                       'face (lurk-get-context-facelist context)
+                       'read-only t
+                       'context context
+                       'invisible context-atom))))))))
 
 (defun lurk-display-message (from to text)
   (let ((context (if (eq 'channel (lurk-get-context-type to))
@@ -457,7 +488,9 @@ portion of the source component of the message, as LURK doesn't use this.")
                    (if (equal to lurk-nick) from to))))
     (lurk-display-string
      context
-     (concat "* " from)
+     (propertize
+      (concat context " * " from)
+      'face (lurk-get-context-facelist context))
      action-text)))
 
 (defun lurk-display-notice (context &rest notices)
@@ -511,17 +544,17 @@ portion of the source component of the message, as LURK doesn't use this.")
        (opt (group (: ":" (+ digit))))
        (opt (group (: "/"
                       (opt
-                       (* (any alnum "-/.,#:%=&_"))
-                       (any alnum "-/#:%=&_")))))))
+                       (* (any alnum "-/.,#:%=&_?~@"))
+                       (any alnum "-/#:%=&_~@")))))))
   "Imperfect regex used to find URLs in plain text.")
 
 (defun lurk-click-url (button)
   (browse-url (button-get button 'url)))
 
-(defun lurk-buttonify-urls (string)
+(defun lurk-buttonify-urls (&rest strings)
   "Turn substrings which look like urls in STRING into clickable buttons."
   (with-temp-buffer
-    (insert string)
+    (apply #'insert strings)
     (goto-char (point-min))
     (while (re-search-forward lurk-url-regex nil t)
       (let ((url (match-string 0)))
@@ -534,6 +567,34 @@ portion of the source component of the message, as LURK doesn't use this.")
                           'help-echo "Open URL in browser.")))
     (buffer-string)))
 
+(defun lurk-add-formatting (string)
+  (with-temp-buffer
+    (insert string)
+    (goto-char (point-min))
+    (let ((bold nil)
+          (italics nil)
+          (underline nil)
+          (strikethrough nil)
+          (prev-point (point)))
+      (while (re-search-forward (rx (any "\x02\x1D\x1F\x1E")) nil t)
+        (let ((beg (+ (match-beginning 0) 1)))
+          (if bold
+              (add-face-text-property prev-point beg '(:weight bold)))
+          (if italics
+              (add-face-text-property prev-point beg '(:slant italic)))
+          (if underline
+              (add-face-text-property prev-point beg '(:underline t)))
+          (if strikethrough
+              (add-face-text-property prev-point beg '(:strike-through t)))
+          (pcase (match-string 0)
+            ("\x02" (setq bold (not bold)))
+            ("\x1D" (setq italics (not italics)))
+            ("\x1F" (setq underline (not underline)))
+            ("\x1E" (setq strikethrough (not strikethrough))))
+          (delete-region (match-beginning 0) (match-end 0))
+          (setq prev-point (point)))))
+    (buffer-string)))
+
 
 ;;; Message evaluation
 ;;
@@ -704,6 +765,14 @@ portion of the source component of the message, as LURK doesn't use this.")
          (setq lurk-debug (not lurk-debug))
          (lurk-display-notice nil "Debug mode now " (if lurk-debug "on" "off") "."))
 
+        ((rx "HEADER")
+         (if header-line-format
+           (progn
+             (setq-local header-line-format nil)
+             (lurk-display-notice nil "Header disabled."))
+           (lurk-setup-header)
+           (lurk-display-notice nil "Header enabled.")))
+
         ((rx (: "CONNECT " (let network (* not-newline))))
          (lurk-display-notice nil "Attempting to connect to " network "...")
          (lurk-connect network))
@@ -742,10 +811,10 @@ portion of the source component of the message, as LURK doesn't use this.")
            (setq lurk-nick nick)
            (lurk-display-notice nil "Set default nick to '" nick "'")))
 
-        ((rx "LIST")
+        ((rx (: "LIST" (* whitespace) string-end))
          (lurk-display-notice nil "This command can generate lots of output. Use `LIST -yes' if you're sure."))
 
-        ((rx (: "LIST" (+ whitespace) "-YES"))
+        ((rx (: "LIST" (+ whitespace) "-YES" (* whitespace) string-end))
          (lurk-send-msg (lurk-msg nil nil "LIST")))
 
         ((rx "MSG "
@@ -835,7 +904,8 @@ portion of the source component of the message, as LURK doesn't use this.")
     (let* ((end (max lurk-input-marker (point)))
            (space-idx (save-excursion
                         (re-search-backward " " lurk-input-marker t)))
-           (start (if space-idx (+ 1 space-idx) lurk-input-marker)))
+           (start (if space-idx (+ 1 space-idx) lurk-input-marker))
+           (completion-ignore-case t))
       (unless (string-prefix-p "/" (buffer-substring start end))
         (completion-in-region start end (lurk-get-context-users lurk-current-context))))))
@@ -1,4 +1,4 @@
-;;; lurk.el --- Little Uni-buffer iRc Klient -*- lexical-binding:t -*-
+;;; lurk.el --- Little Unibuffer iRc Klient -*- lexical-binding:t -*-
 
 ;; Copyright (C) 2021 Tim Vaughan